#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <rsbac/types.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include <rsbac/aci_data_structures.h>

const char   set_prog[] = "attr_set_file_dir";
enum rsbac_attribute_t attr_list[RSBAC_FD_NR_ATTRIBUTES] = RSBAC_FD_ATTR_LIST;
int recurse = 0;
int verbose = 0;

int process(char * name, FILE * tfile)
  {
    struct rsbac_file_dir_aci_t default_fd_aci = DEFAULT_FD_ACI;
    int def_attr[RSBAC_FD_NR_ATTRIBUTES] = {
      default_fd_aci.sec_level,
      default_fd_aci.object_category,
      default_fd_aci.data_type,
      default_fd_aci.program_type,
      default_fd_aci.pm_object_class,
      default_fd_aci.pm_tp,
      default_fd_aci.pm_object_type,
      default_fd_aci.ms_scanned,
      default_fd_aci.ms_trusted
      };
    int res = 0;
    char tmp1[120];
    char tmp2[65];
    enum rsbac_target_t type;
    char type_n[5] = "";
    int j;
    struct stat buf;
    union rsbac_attribute_value_t value;

    if (lstat(name,&buf))
      {
        fprintf(stderr, "stat for file %s returned error: %s\n",
               name,
               strerror(errno));
        return(1);
      }
    if(S_ISDIR(buf.st_mode))
      {
        type = T_DIR;
        strcpy(type_n,"DIR");
      }
    else
      if(   (S_ISREG(buf.st_mode))
         || (S_ISBLK(buf.st_mode))
         || (S_ISCHR(buf.st_mode)) )
        {
          type = T_FILE;
          strcpy(type_n,"FILE");
        }
      else
        {
          if(verbose)
            printf("%s is neither file nor dir, skipped\n",
                   name);
          return(-1);
        }
    if(verbose)
      printf("Processing %s '%s'\n", type_n, name);
    for (j=0;j < RSBAC_FD_NR_ATTRIBUTES;j++)
      {
        value.dummy = -1;
        res = rsbac_get_attr_n(type, name, attr_list[j], &value);
        if(res)
          {
            if((errno<RSBAC_EPERM) || (errno > RSBAC_EDECISIONMISMATCH))
              fprintf(stderr, "%s (%s): %s\n",
                      name,
                      get_attribute_name(tmp1,attr_list[j]),
                      strerror(errno));
            else
              {
                get_error_name(tmp1,errno);
                fprintf(stderr, "%s (%s): %s\n",
                        name,
                        get_attribute_name(tmp2,attr_list[j]),
                        tmp1);
              }
          }
        else
          if(   attr_list[j] == A_log_array_low
             || attr_list[j] == A_log_array_high)
            {
              if (value.log_array_low != -1)
                fprintf(tfile,
                        "%s %s %s %s %s\n",
                        set_prog,
                        type_n,
                        name,
                        get_attribute_name(tmp1,attr_list[j]),
                        u64tostrlog(tmp2,value.log_array_low));
            }
          else
            if(value.dummy != def_attr[j])
              fprintf(tfile,
                      "%s %s %s %s %i\n",
                      set_prog,
                      type_n,
                      name,
                      get_attribute_name(tmp1,attr_list[j]),
                      value.dummy);
      }
    if (   (type == T_DIR)
        && recurse)
      {
        DIR * dir_stream_p;
        struct dirent * dirent_p;
        char name2[256];

        if(S_ISLNK(buf.st_mode))
          return(0);
        if(!(dir_stream_p = opendir(name)))
          {
            fprintf(stderr, "opendir for dir %s returned error: %s\n",
                   name,
                   strerror(errno));
            return(-2);
          }
        while((dirent_p = readdir(dir_stream_p)))
          {
            if(   (strcmp(".",dirent_p->d_name))
               && (strcmp("..",dirent_p->d_name)) )
              {
                strcpy(name2,name);
                strcat(name2,"/");
                strcat(name2,dirent_p->d_name);
                process(name2, tfile);
              }
          }
        closedir(dir_stream_p);
      }
    return(0);
  }

int main(int argc, char ** argv)
{
  int res = 0;
  char tmp1[80],tmp2[120];
  int i,j;
  FILE * tfile;
  char * progname;

  progname = argv[0];
  if ((argc > 1) && (!strcmp("-r",argv[1])) )
    {
      recurse=1;
      argv++;
      argc--;
    }  
  if ((argc > 1) && (!strcmp("-v",argv[1])) )
    {
      verbose=1;
      argv++;
      argc--;
    }  
  if (argc > 2)
    {
      if (!(tfile=fopen(argv[1],"w")))
        {
          fprintf(stderr, "opening target file returned error: %s\n", strerror(errno));
        }
      printf("%s: %i targets", progname, argc - 2);
      if(recurse)
        printf(" - recursing");
      printf("\n");
      for (i=1;i < (argc-1);i++)
        {
          process(argv[i+1],tfile);
        }
      fclose(tfile);
    }
  else
    {
      printf("\nUse: %s [-r] [-v] target-file file/dirname(s)\n", progname);  
      printf("- should be called by root with all rsbac modules switched off,\n");
      printf("- -r = recurse in subdirs, -v = verbose, no symlinks followed,\n");
      printf("- target-file = name of restore script to be created,\n");
      printf("- attr_listibutes and values in backup = see following list:\n");
      for (j=0;j<RSBAC_FD_NR_ATTRIBUTES;j++)
        {
          get_attribute_name(tmp1,attr_list[j]);
          get_attribute_param(tmp2,attr_list[j]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
    }
  return (res);
}

